@@ -114,10 +114,9 @@ ns_new_resolve(JSContext *context,
                 goto out;
             }
         } else {
-            gjs_throw(context,
-                      "No symbol '%s' in namespace '%s'",
-                      name, priv->namespace);
+            /* No property defined, but no error either, so return TRUE */
             JS_EndRequest(context);
+            ret = JS_TRUE;
             goto out;
         }
     }
@@ -156,9 +155,11 @@ ns_new_resolve(JSContext *context,
  */
 GJS_NATIVE_CONSTRUCTOR_DECLARE(ns)
 {
-    JSObject *object = NULL;                            \
+    GJS_NATIVE_CONSTRUCTOR_VARIABLES(ns)
     Ns *priv;
 
+    (void) argv;
+
     GJS_NATIVE_CONSTRUCTOR_PRELUDE(ns);
 
     priv = g_slice_new0(Ns);
@@ -286,23 +287,9 @@ ns_new(JSContext    *context,
 }
 
 JSObject*
-gjs_define_ns(JSContext    *context,
-              JSObject     *in_object,
+gjs_create_ns(JSContext    *context,
               const char   *ns_name,
               GIRepository *repo)
 {
-    JSObject *ns;
-
-    ns = ns_new(context, ns_name, repo);
-
-    if (!JS_DefineProperty(context, in_object,
-                           ns_name, OBJECT_TO_JSVAL(ns),
-                           NULL, NULL,
-                           GJS_MODULE_PROP_FLAGS))
-        gjs_fatal("no memory to define ns property");
-
-    gjs_debug(GJS_DEBUG_GNAMESPACE,
-              "Defined namespace '%s' %p in GIRepository %p", ns_name, ns, in_object);
-
-    return ns;
+    return ns_new(context, ns_name, repo);
 }
