@@ -47,11 +47,18 @@ setup(GjsTestJSFixture *fix,
     int code;
     char *filename;
     char *search_path[2];
+    const char *test_filename = test_data;
+    const char *js_version;
+
+    js_version = gjs_context_scan_file_for_js_version(test_filename);
 
     search_path[0] = g_build_filename(top_srcdir, "test", "modules", NULL);
     search_path[1] = NULL;
 
-    fix->context = gjs_context_new_with_search_path(search_path);
+    fix->context = g_object_new (GJS_TYPE_CONTEXT,
+                                 "search-path", search_path,
+                                 "js-version", js_version,
+                                 NULL);
     g_free(search_path[0]);
 
     /* Load jsUnit.js directly into global scope, rather than
@@ -111,10 +118,57 @@ read_all_dir_sorted (const char *dirpath)
     return result;
 }
 
+/* Always return an absolute filename and treat all absolute path elements as
+ * absolute, not just the first one, e.g.
+ *     ("/foo", "/bar", NULL) returns "/bar"
+ * (g_build_filename would return "/foo/bar")
+ * Returned path may still include '..' path elements.
+ */
+static char *
+build_absolute_filename(const char *first_element,
+                        ...)
+{
+    const char *element;
+    va_list ap;
+    char *cwd;
+    char *path;
+    GPtrArray *array;
+
+    cwd = g_get_current_dir();
+
+    array = g_ptr_array_new();
+
+    g_ptr_array_add(array, cwd);
+
+    va_start(ap, first_element);
+
+    element = first_element;
+    while (element != NULL) {
+        if (g_path_is_absolute(element))
+            g_ptr_array_set_size(array, 0);
+
+        g_ptr_array_add(array, (char*)element);
+
+        element = va_arg(ap, const char *);
+    }
+    va_end(ap);
+
+    g_ptr_array_add(array, NULL);
+
+    path = g_build_filenamev((char **)array->pdata);
+
+    g_ptr_array_free(array, TRUE);
+
+    g_free(cwd);
+
+    return path;
+}
+
 int
 main(int argc, char **argv)
 {
-    /* These are relative to top_builddir */
+    /* These may be absolute or relative to top_builddir, depending whether
+     * GJS_TOP_SRCDIR is absolute or not */
     const char * const path_directories[] = {
         GJS_TOP_SRCDIR"/modules",
         GJS_TOP_SRCDIR"/test/js/modules",
@@ -131,18 +185,19 @@ main(int argc, char **argv)
     GString *path;
     size_t i;
     GSList *all_tests, *iter;
+    GSList *test_filenames = NULL;
+    int retval;
 
     working_dir = g_get_current_dir();
 
-    if(g_path_is_absolute(argv[0]))
-        gjs_unit_path = g_strdup(argv[0]);
-    else
-        gjs_unit_path = g_build_filename(working_dir, argv[0], NULL);
+    gjs_unit_path = build_absolute_filename(argv[0], NULL);
 
     gjs_unit_dir = g_path_get_dirname(gjs_unit_path);
+    g_free(gjs_unit_path);
     /* the gjs-unit executable will be in <top_builddir>/.libs */
-    top_builddir = g_build_filename(gjs_unit_dir, "..", NULL);
-    top_srcdir = g_build_filename(top_builddir, GJS_TOP_SRCDIR, NULL);
+    top_builddir = g_path_get_dirname(gjs_unit_dir);
+    g_free(gjs_unit_dir);
+    top_srcdir = build_absolute_filename(top_builddir, GJS_TOP_SRCDIR, NULL);
 
     /* Normalize, not strictly necessary */
     g_chdir(top_builddir);
@@ -154,6 +209,7 @@ main(int argc, char **argv)
     top_srcdir = g_get_current_dir();
 
     g_chdir(working_dir);
+    g_free(working_dir);
 
     /* we're always going to use uninstalled files, set up necessary
      * environment variables, but don't overwrite if already set */
@@ -166,15 +222,18 @@ main(int argc, char **argv)
         if (i != 0)
             g_string_append_c(path, ':');
 
-        directory = g_build_filename(top_builddir, path_directories[i], NULL);
+        directory = build_absolute_filename(top_builddir, path_directories[i], NULL);
         g_string_append(path, directory);
         g_free(directory);
     }
 
     g_setenv("TOP_SRCDIR", top_srcdir, FALSE);
     g_setenv("BUILDDIR", top_builddir, FALSE);
+    g_free(top_builddir);
     g_setenv("XDG_DATA_HOME", data_home, FALSE);
+    g_free(data_home);
     g_setenv("GJS_PATH", path->str, FALSE);
+    g_string_free(path, TRUE);
     /* The tests are known to fail in the presence of the JIT;
      * we leak objects.
      * https://bugzilla.gnome.org/show_bug.cgi?id=616193
@@ -189,7 +248,6 @@ main(int argc, char **argv)
                 gjs_crash_after_timeout(timeout);
         }
     }
-    gjs_init_sleep_on_crash();
 
     setlocale(LC_ALL, "");
     g_test_init(&argc, &argv, NULL);
@@ -206,8 +264,10 @@ main(int argc, char **argv)
         char *file_name;
 
         if (!(g_str_has_prefix(name, "test") &&
-              g_str_has_suffix(name, ".js")))
+              g_str_has_suffix(name, ".js"))) {
+            g_free(name);
             continue;
+        }
 
         /* pretty print, drop 'test' prefix and '.js' suffix from test name */
         test_name = g_strconcat("/js/", name + 4, NULL);
@@ -217,9 +277,15 @@ main(int argc, char **argv)
         g_test_add(test_name, GjsTestJSFixture, file_name, setup, test, teardown);
         g_free(name);
         g_free(test_name);
-        /* not freeing file_name as it's needed while running the test */
+        test_filenames = g_slist_prepend(test_filenames, file_name);
+        /* not freeing file_name yet as it's needed while running the test */
     }
+    g_free(js_test_dir);
     g_slist_free(all_tests);
 
-    return g_test_run ();
+    retval =  g_test_run ();
+    g_slist_foreach(test_filenames, (GFunc)g_free, test_filenames);
+    g_slist_free(test_filenames);
+    g_free(top_srcdir);
+    return retval;
 }
